@@ -185,77 +185,83 @@ def get_feature_group(location: str):
         group = feature_mapping.get(word)
         if group:
             return word, group
-        
+
     return '', ''
 
+
 feature_list = ["ADM1", "ADM2", "ADM3", "ADM4", "ADMF", "AREA", "CH", "CSTL", "CMTY", "EST ", "HSP", "FT",
-           "HSTS", "ISL", "MSQE", "MSTY", "MT", "MUS", "PAL", "PPL", "PPLA", "PPLA2", "PPLA3", "PPLA4",
-           "PPLC", "PPLG", "PPLH", "PPLL", "PPLQ", "PPLX", "PRK", "PRN", "PRSH", "RUIN", "RGN", "RLG", "STG",
-           "SQR", "SYG", "VAL", "MNMT"]
+                "HSTS", "ISL", "MSQE", "MSTY", "MT", "MUS", "PAL", "PPL", "PPLA", "PPLA2", "PPLA3", "PPLA4",
+                "PPLC", "PPLG", "PPLH", "PPLL", "PPLQ", "PPLX", "PRK", "PRN", "PRSH", "RUIN", "RGN", "RLG", "STG",
+                "SQR", "SYG", "VAL", "MNMT"]
 
 feature_mapping = {
     # If location has item below, then derive its feature type
-    'st'           : 'CH',
-    'kapelle'      : 'CH',
-    'chapel'       : 'CH',
-    'kirche'       : 'CH',
-    'kirke'        : 'CH',
-    'church'       : 'CH',
-    'eglise'       : 'CH',
-    'cathedral'    : 'CH',
-    'cathedrale'   : 'CH',
-    'kathedrale'   : 'CH',
-    'kanisat'      : 'CH',
-    'iglesia'      : 'CH',
-    'catedral'     : 'CH',
-    'parroquia'    : 'CH',
-    'chapelle'     : 'CH',
-    'chiesa'       : 'CH',
-    'cappella'     : 'CH',
-    'basilica'     : 'CH',
-    'cattedrale'   : 'CH',
+    'st'                            : 'CH',
+    'kapelle'                       : 'CH',
+    'chapel'                        : 'CH',
+    'kirche'                        : 'CH',
+    'kirke'                         : 'CH',
+    'church'                        : 'CH',
+    'eglise'                        : 'CH',
+    'cathedral'                     : 'CH',
+    'cathedrale'                    : 'CH',
+    'kathedrale'                    : 'CH',
+    'kanisat'                       : 'CH',
+    'iglesia'                       : 'CH',
+    'catedral'                      : 'CH',
+    'parroquia'                     : 'CH',
+    'chapelle'                      : 'CH',
+    'chiesa'                        : 'CH',
+    'cappella'                      : 'CH',
+    'basilica'                      : 'CH',
+    'cattedrale'                    : 'CH',
 
-    'castle'       : 'CSTL%',
-    'schloss'      : 'CSTL%',
-    'chateau'      : 'CSTL%',
-    'castell'      : 'CSTL%',
-    'castillo'     : 'CSTL%',
-    'castello'     : 'CSTL%',
-    'kasteel'      : 'CSTL%',
-    'zamek'        : 'CSTL%',
-    'slott'        : 'CSTL%',
+    'castle'                        : 'CSTL%',
+    'schloss'                       : 'CSTL%',
+    'chateau'                       : 'CSTL%',
+    'castell'                       : 'CSTL%',
+    'castillo'                      : 'CSTL%',
+    'castello'                      : 'CSTL%',
+    'kasteel'                       : 'CSTL%',
+    'zamek'                         : 'CSTL%',
+    'slott'                         : 'CSTL%',
 
-    'cemetery'     : 'CMTY',
-    'cimetiere'    : 'CMTY',
-    'cementerio'   : 'CMTY',
-    'cimitero'     : 'CMTY',
+    'cemetery'                      : 'CMTY',
+    'cimetiere'                     : 'CMTY',
+    'cementerio'                    : 'CMTY',
+    'cimitero'                      : 'CMTY',
     'rooms katholieke begraafplaats': 'CMTY',
-    'begraafplaats': 'CMTY',
-    'gravlund'     : 'CMTY',
-    'kirkegard'    : 'CMTY',
-    'friedhof'     : 'CMTY',
+    'begraafplaats'                 : 'CMTY',
+    'gravlund'                      : 'CMTY',
+    'kirkegard'                     : 'CMTY',
+    'friedhof'                      : 'CMTY',
 
-    'hospital'     : 'HSP',
-    'hopital'      : 'HSP',
-    'krankenhaus'  : 'HSP',
-    'ospedale'     : 'HSP',
+    'hospital'                      : 'HSP',
+    'hopital'                       : 'HSP',
+    'krankenhaus'                   : 'HSP',
+    'ospedale'                      : 'HSP',
 
-    'palace'        : 'PAL',
-    'palais': 'PAL',
+    'palace'                        : 'PAL',
+    'palais'                        : 'PAL',
 
-    'county'       : 'ADM2',
-    'island'       : 'ISL',
+    'county'                        : 'ADM2',
+    
+    'island'                        : 'ISL',
+    'isle'                          : 'ISL',
+    'ile'                           : 'ISL',
+    'ostrov'                        : 'ISL',
+    'islet'                         : 'ISL',
 
-    'mountain'     : "MT",
-    'museum'       : "MUS",
-    'mosque'       : "MSQE",
-    'monastery'    : "MSTY",
-    'park'         : "PRK",
-    'prison'       : "PRN",
-    'square'       : "SQR",
-    'synagogue'    : "SYG",
-    'abbey'         : "MSTY%",
-    'priory'        : "MSTY%",
+    'mountain'                      : "MT",
+    'museum'                        : "MUS",
+    'mosque'                        : "MSQE",
+    'monastery'                     : "MSTY",
+    'park'                          : "PRK",
+    'prison'                        : "PRN",
+    'square'                        : "SQR",
+    'synagogue'                     : "SYG",
+    'abbey'                         : "MSTY%",
+    'priory'                        : "MSTY%",
     }
 
 feature_names = {
