@@ -18,7 +18,8 @@
 #   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 """
 Provide functions to normalize text strings by converting to lowercase, removing noisewords.   
-This is used by the lookup functions and the database build functions and match scoring   
+This is used by the lookup functions and the database build functions and match scoring  
+ 
 noise_words is a list of replacements only used for match scoring   
 phrase_cleanup is a list of replacements for db build, lookup and match scoring   
 """
@@ -29,14 +30,45 @@ import unidecode
 
 from geodata import GeodataBuild, Loc, GeoUtil
 
-# Todo -  make these all list driven
+# Todo -  make all of these list driven
 
-noise_words = [
-    # Noise words - Replacements done for match scoring
+
+stop_words = {
+    # list of stop words
+    # These are forced to sort at end of Soundex name so that search for 'north pinewood' can be done as 'pinewood%' and
+    # will find 'pinewood north'
+    'east',
+    'west',
+    'north',
+    'south',
+    'the',
+    'of',
+    'town',
+    'city',
+    'county',
+    'and',
+    'on',
+    'by',
+    'new',
+    'old',
+    'big',
+    'little',
+    'royal',
+    'borough',
+    'department'
+    }
+
+scoring_noise_words = [
+    # list of substitutions applied to text for match scoring
+    # list of (Regex, Replacement) - re.sub applied 
     (r', ', ','),
     (r"normandy american ", 'normandie american '),
     (r'nouveau brunswick', ' '),
     (r'westphalia', 'westfalen'),
+    (r' departement', ' department'),
+    (r'royal borough of windsor and maidenhead', 'berkshire'),
+    (r'regional municipality', 'county'),
+    (r'kathedrale', 'cathedral'),
     (r"l'", ''),
     (r'citta metropolitana di ', ' '),
     (r'kommune', ''),
@@ -53,13 +85,14 @@ noise_words = [
     (r' di ', ' '),
     (r' du ', ' '),
     (r' of ', ' '),
-    (r' departement', ' '),
-    (r'royal borough of windsor and maidenhead', 'berkshire'),
-    (r'regional municipality', 'county'),
     ]
 
+
 phrase_cleanup = [
-    # Phrase cleanup - replacements that are always applied (for database build, lookup, and match scoring)  
+    # Phrase cleanup - replacements applied for database build, lookup, and match score 
+    # list of (Regex, Replacement) - re.sub applied 
+    ('  +', ' '),  # Strip multiple space to single space
+    (r'^mt ', 'mount '),
     ('r\.k\. |r k ', 'roman catholic '),
     ('rooms katholieke ', 'roman catholic '),
     ('sveti |saints |sainte |sint |saint |sankt |st\. ', 'st '),  # Normalize Saint to St
@@ -68,96 +101,43 @@ phrase_cleanup = [
     (r'united states', 'usa'),  # Normalize to USA
     (r'town of ', ''),  # - remove town of
     (r'city of ', ''),  # - remove city of
-    (r'near ', ' '),  # - remove near
     (r'cimetiere', 'cemetery'),  # cimetière
     (r'begraafplaats', 'cemetery'),  # 
-    ('  +', ' '),  # Strip multiple space
-    ('([^,]+) of ([^,]+)', r'\g<2> g<1>'),  # Normalize 'XYZ of X' to 'X XYZ'. eg. 'Township of X' to 'X Township'
-    #('township of ([^,]+)', r'\g<1> township'),  # Normalize 'Township of X' to 'X Township'
-    #('cathedral of ([^,]+)', r'\g<1> cathedral'),  # Normalize 'cath of X' to 'X cath'
-    #('palace of ([^,]+)', r'\g<1> palace'),  # Normalize 'palace of X' to 'X palace'
-    #('castle of ([^,]+)', r'\g<1> castle'),  # Normalize 'castle of X' to 'X castle'
-    #(r',castle', ' castle'),  # -  remove extra comma
-    #(r',palace', ' palace'),  # -  remove extra comma
-    (r"'(\w{2,})'",r"\g<1>"), # remove single quotes around word
+    ('county of ([^,]+)', r'\g<1> county'),  # Normalize 'Township of X' to 'X Township'
+    ('township of ([^,]+)', r'\g<1> township'),  # Normalize 'Township of X' to 'X Township'
+    ('cathedral of ([^,]+)', r'\g<1> cathedral'),  # Normalize 'Township of X' to 'X Township'
+    ('palace of ([^,]+)', r'\g<1> palace'),  # Normalize 'Township of X' to 'X Township'
+    ('castle of ([^,]+)', r'\g<1> castle'),  # Normalize 'Township of X' to 'X Township'
+    (r',castle', ' castle'),  # -  remove extra comma
+    (r',palace', ' palace'),  # -  remove extra comma
+    (r"'(\w{2,})'", r"\g<1>"),  # remove single quotes around word, but leave apostrophe
     ]
 
 
-def normalize_for_scoring(text: str, iso: str) -> str:
-    """
-        Normalize the title we use to determine how close a match we got. 
-        See normalize() for details  
-        Also remove noise words such as City Of
-
-    #Args:
-        text: text to normalize
-        iso: ISO country code
-
-    #Returns:
-
-    """
-    text = normalize(text=text, remove_commas=False)
-    text = _remove_noise_words(text)
-    return text
-
-
-def normalize(text: str, remove_commas: bool) -> str:
-    """
-    Normalize text - Convert from UTF-8 to lowercase ascii.  
-    Remove commas if parameter set.   
-    Remove all non alphanumeric except $ and *  
-    Then call _phrase_normalize() which normalizes common phrases with multiple spellings, such as saint to st   
-    #Args:   
-        text:  Text to normalize   
-        remove_commas:   True if commas should be removed   
-
-    #Returns:   
-        Normalized text
-
-    """
-
-    # Convert UT8 to ascii
-    text = unidecode.unidecode(text)
-    text = str(text).lower()
-
-    # remove all non alphanumeric except $ and * and comma(if flag set)
-    if remove_commas:
-        text = re.sub(r"[^a-z0-9 $*']+", " ", text)
-    else:
-        text = re.sub(r"[^a-z0-9 $*,']+", " ", text)
-
-    text = _phrase_normalize(text)
-    return text.strip(' ')
-
-
-def _phrase_normalize(text: str) -> str:
-    """ Strip spaces and normalize spelling for items such as Saint and County """
-    # Replacement patterns to clean up entries
-
-    if 'amt' not in text:
-        text = re.sub(r'^mt ', 'mount ', text)
-
-    for pattern, replace in phrase_cleanup:
-        text = re.sub(pattern, replace, text)
-
-    return text
-
-
-def _remove_noise_words(text: str):
-    # Calculate score with noise words removed    
-    for pattern, replace in noise_words:
-        text = re.sub(pattern, replace, text)
-    return text
-
-
-def remove_aliase(input_words, res_words) -> (str, str):
-    if "middlesex" in input_words and "greater london" in res_words:
-        input_words = re.sub('middlesex', '', input_words)
-        res_words = re.sub('greater london', '', res_words)
-    return input_words, res_words
+local_country_names = {
+    # Dictionary of key=local name, val = english name for country
+    'norge'       : 'norway',
+    'sverige'     : 'sweden',
+    'osterreich'  : 'austria',
+    'belgie'      : 'belgium',
+    'brasil'      : 'brazil',
+    'danmark'     : 'denmark',
+    'eire'        : 'ireland',
+    'magyarorszag': 'hungary',
+    'italia'      : 'italy',
+    'espana'      : 'spain',
+    'deutschland' : 'germany',
+    'prussia'     : 'germany',
+    'suisse'      : 'switzerland',
+    'schweiz'     : 'switzerland',
+    }
 
+ALIAS_FEAT = 2
+ALIAS_ISO = 1
+ALIAS_NAME = 0
 
 alias_list = {
+    # Dictionary of key=local name, val = (english name, iso, feature) for country, province, county
     'norge'               : ('norway', '', 'ADM0'),
     'sverige'             : ('sweden', '', 'ADM0'),
     'osterreich'          : ('austria', '', 'ADM0'),
@@ -195,38 +175,59 @@ alias_list = {
     'breconshire'         : ('sir powys', 'gb', 'ADM2'),
     }
 
-ALIAS_FEAT = 2
-ALIAS_ISO = 1
-ALIAS_NAME = 0
+
+def normalize(text: str, remove_commas: bool) -> str:
+    """
+    Normalize text - Convert from UTF-8 to lowercase ascii.  
+    Remove commas if parameter set.   
+    Remove all non alphanumeric except $ and *  
+    Then call _phrase_normalize() which normalizes common phrases with multiple spellings, such as saint to st   
+    #Args:   
+        text:  Text to normalize   
+        remove_commas:   True if commas should be removed   
+
+    #Returns:   
+        Normalized text
+
+    """
+    # Convert UT8 to ascii
+    text = unidecode.unidecode(text)
+    text = str(text).lower()
+
+    # remove all non alphanumeric except $ and * and comma(if flag set)
+    if remove_commas:
+        text = re.sub(r"[^a-z0-9 $*']+", " ", text)
+    else:
+        text = re.sub(r"[^a-z0-9 $*,']+", " ", text)
+
+    text = _phrase_normalize(text)
+    return text.strip(' ')
 
 
-def add_aliases_to_db(geo_build: GeodataBuild):
-    #  Add alias names to DB
-    for ky in alias_list:
-        add_alias_to_db(ky, geo_build)
+def normalize_for_scoring(text: str, iso: str) -> str:
+    """
+        Normalize the text we use to determine how close a match we got. 
+
+    #Args:
+        text: text to normalize
+        iso: ISO country code
+
+    #Returns:
+
+    """
+    text = normalize(text=text, remove_commas=False)
+    text = _remove_noise_words(text)
+    return text
 
 
-def add_alias_to_db(ky: str, geo_build: GeodataBuild):
-    alias_row = alias_list.get(ky)
-    place = Loc.Loc()
-    place.country_iso = alias_row[ALIAS_ISO].lower()
-    place.city = alias_row[ALIAS_NAME]
-    place.feature = alias_row[ALIAS_FEAT]
-    place.place_type = Loc.PlaceType.CITY
+def _phrase_normalize(text: str) -> str:
+    """ Strip spaces and normalize spelling for items such as Saint and County """
+    # Replacement patterns to clean up entries
 
-    # Lookup main entry and get GEOID
-    geo_build.geodb.s.lookup_place(place)
-    if len(place.georow_list) > 0:
-        if len(place.georow_list[0]) > 0:
-            geo_row = list(place.georow_list[0][0:GeoUtil.Entry.SDX+1])
-            geo_build.update_geo_row_name(geo_row=geo_row, name=ky)
-            geo_tuple = tuple(geo_row)
-            geo_build.insert(geo_tuple=geo_tuple, feat_code=alias_row[ALIAS_FEAT])
+    for pattern, replace in phrase_cleanup:
+        text = re.sub(pattern, replace, text)
 
-
-def deb(msg=None):
-    # Display debug message with line number
-    print(f"Debug {sys._getframe().f_back.f_lineno}: {msg if msg is not None else ''}")
+    return text
 
 
 def admin1_normalize(admin1_name: str, iso):
@@ -290,33 +291,17 @@ def country_normalize(country_name) -> (str, bool):
     """
     country_name = re.sub(r'\.', '', country_name)  # remove .
 
-    natural_names = {
-        'norge'       : 'norway',
-        'sverige'     : 'sweden',
-        'osterreich'  : 'austria',
-        'belgie'      : 'belgium',
-        'brasil'      : 'brazil',
-        'danmark'     : 'denmark',
-        'eire'        : 'ireland',
-        'magyarorszag': 'hungary',
-        'italia'      : 'italy',
-        'espana'      : 'spain',
-        'deutschland' : 'germany',
-        'prussia'     : 'germany',
-        'suisse'      : 'switzerland',
-        'schweiz'     : 'switzerland',
-
-        }
-    if natural_names.get(country_name):
-        country_name = natural_names.get(country_name)
+    if local_country_names.get(country_name):
+        country_name = local_country_names.get(country_name)
         return country_name.strip(' '), True
     else:
         return country_name.strip(' '), False
 
-def normalize_features(feature, name, pop:int):
+
+def feature_normalize(feature, name, pop: int):
     # Set city feature based on population and cleanup features for RUIN and HSTS
     feat = feature
-    
+
     # Create new Feature codes based on city population
     if pop > 1000000 and 'PP' in feature:
         feat = 'PP1M'
@@ -324,23 +309,91 @@ def normalize_features(feature, name, pop:int):
         feat = 'P1HK'
     elif pop > 10000 and 'PP' in feature:
         feat = 'P10K'
-    
+
     if feature == 'AREA':
         if 'island' in name:
             feat = 'ISL'
-    
-    # Set feature type for Abbey/Priory, Castle, and Church if RUIN or HSTS
+
+    # Set feature type for Abbey/Priory, Castle, and Church if feature is RUIN or HSTS and name matches
     if feature == 'HSTS' or feature == 'RUIN':
-        if 'abbey' in name or 'priory' in name:
-            # Change feature to MSTY plus first letter of original feature code (H or R)
-            feat = 'MSTY' + feature[0]
+        if 'abbey' in name :
+            feat = 'MSTY' 
         elif 'priory' in name:
-            # Change feature to MSTY plus first letter of original feature code
-            feat = 'MSTY' + feature[0]
+            feat = 'MSTY' 
         elif 'castle' in name:
-            # Change feature to MSTY plus first letter of original feature code
-            feat = 'CSTL' + feature[0]
+            feat = 'CSTL'
         elif 'church' in name:
-            # Change feature to MSTY plus first letter of original feature code
-            feat = 'CH_' + feature[0]
+            feat = 'CH' 
     return feat
+
+
+def stop_words_last(item):
+    """
+    key function for sorting - return key but force stop words to sort last
+    Stop words come from Normalize.stop_words
+    Args:
+        item: 
+
+    Returns:
+        key for item, with stop words forced last
+    """
+    if item in stop_words:
+        # Force stop words to sort last.  { will sort last
+        return '{' + item
+    return item
+
+
+def sorted_normalize(text):
+    # Remove l' and d'  
+    text = re.sub(r"l\'", "", text)
+    text = re.sub(r"d\'", "", text)
+
+    # Modify phrase X of Y to be Y X.  County of Whitley becomes Whitley County
+    text = re.sub('([^,]+) of ([^,]+)', r'\g<2> \g<1>', text)
+
+    # Force stop words to sort at end of list so wildcard will find phrases without them  (East, West, North, South)
+    # North Haden becomes Haden North and Haden% will match it.
+    word_list = sorted(text.split(' '), key=stop_words_last)
+    return word_list
+
+
+def _remove_noise_words(text: str):
+    # Calculate score with noise words removed    
+    for pattern, replace in scoring_noise_words:
+        text = re.sub(pattern, replace, text)
+    return text
+
+
+def remove_aliase(input_words, res_words) -> (str, str):
+    if "middlesex" in input_words and "greater london" in res_words:
+        input_words = re.sub('middlesex', 'greater london', input_words)
+    return input_words, res_words
+
+
+def add_aliases_to_db(geo_build: GeodataBuild):
+    #  Add alias names to DB
+    for ky in alias_list:
+        add_alias_to_db(ky, geo_build)
+
+
+def add_alias_to_db(ky: str, geo_build: GeodataBuild):
+    alias_row = alias_list.get(ky)
+    place = Loc.Loc()
+    place.country_iso = alias_row[ALIAS_ISO].lower()
+    place.city = alias_row[ALIAS_NAME]
+    place.feature = alias_row[ALIAS_FEAT]
+    place.place_type = Loc.PlaceType.CITY
+
+    # Lookup main entry and get GEOID
+    geo_build.geodb.s.lookup_place(place)
+    if len(place.georow_list) > 0:
+        if len(place.georow_list[0]) > 0:
+            geo_row = list(place.georow_list[0][0:GeoUtil.Entry.SDX + 1])
+            geo_build.update_geo_row_name(geo_row=geo_row, name=ky)
+            geo_tuple = tuple(geo_row)
+            geo_build.insert(geo_tuple=geo_tuple, feat_code=alias_row[ALIAS_FEAT])
+
+def deb(msg=None):
+    # Display debug message with line number
+    print(f"Debug {sys._getframe().f_back.f_lineno}: {msg if msg is not None else ''}")
+
